#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define CNT_P 15
#define CNT_Q 100

#define A         -100
#define B          100
#define SIGMA_INI  3.0
#define TAU_0      1/(sqrt(2*sqrt(2)))
#define TAU        1/(sqrt(2*2      ))

typedef struct {
  double x1, x2;
  double sig1, sig2;
  double f;
} CHROMOZOM;

CHROMOZOM P[CNT_P];           // populacia rodicov
CHROMOZOM Q[CNT_Q];           // populacia potomkov

double f(double x1, double x2)
{
  double a, b;

  a = x2 - x1*x1;
  b = x1 -1;
  return 100*a*a + b*b;
}

double gaussdev() {

  double v1, v2, fac;
  double r = 2.0;

  while (r >= 1.0) {

    v1 = rand();
    v1 = 2.0*(v1/RAND_MAX) - 1.0;
    v2 = rand();
    v2 = 2.0*(v2/RAND_MAX) - 1.0;
    r = v1*v1 + v2*v2;
  }

  fac = sqrt(-2.0*log(r)/r);
  return fac*v1;
}

double perturbation(double x)
{
  while ((x > B) || (x < A)) {
    if (x > B) x = 2*B - x;
    if (x < A) x = 2*A - x;
  }
  return x;
}

void P_init()
{
  int i;

  for (i = 0; i < CNT_P; i++) {
    P[i].x1   = P[i].x2   = random(201) - 101;
    P[i].sig1 = P[i].sig2 = SIGMA_INI;
    P[i].f    = f(P[i].x1, P[i].x2);
  }
}

CHROMOZOM kriz()
{
  int r[1];
  CHROMOZOM novy;
  double N_0_1_glob;
  double delta_x1, delta_x2, d_x1, d_x2;

  r[0] = random(CNT_P);
  while (r[0] == (r[1] = random(CNT_P)));

  novy.x1   = P[r[random(1)]].x1;
  novy.x2   = P[r[random(1)]].x2;
  novy.sig1 = (P[r[0]].sig1 + P[r[1]].sig1)/2;
  novy.sig2 = (P[r[0]].sig2 + P[r[1]].sig2)/2;

  N_0_1_glob = gaussdev();

  novy.sig1 *= exp(TAU_0*N_0_1_glob) * exp(TAU*gaussdev());
  novy.sig2 *= exp(TAU_0*N_0_1_glob) * exp(TAU*gaussdev());
  d_x1  =  novy.sig1 * gaussdev();
  d_x2  =  novy.sig2 * gaussdev();
  delta_x1  =  d_x1;
  delta_x2  =  d_x2;
  novy.x1   = perturbation(novy.x1 + delta_x1);
  novy.x2   = perturbation(novy.x2 + delta_x2);

  novy.f    = f(novy.x1,novy.x2);

  return novy;
}

void nova_populacia() {
  int j;

  for (j = 0; j < CNT_Q; j++) {
    Q[j] = kriz();
  }
}

void nahrad_populaciu() {
  int i, j;
  int min;

  for (i = 0; i < CNT_P; i++) {
    min = i;
    for (j = i+1; j < CNT_Q; j++) {
      if (Q[j].f < Q[min].f) min = j;
    }
    P[i] = Q[min];
    Q[min] = Q[i];
  }
}

void main() {

  int t, i;
  double sum = 0;

  randomize();
  for (i = 0; i < 100; i++) {
    P_init();
    P[0].f = 1;

    t = 0;
    while (P[0].f >= 0.000001) {
      t++;
      nova_populacia();
      nahrad_populaciu();
    }
    sum += t;
  }

  printf("%f %f %f ",P[0].x1, P[0].x2, P[0].f);
  printf("%0.2f\n",sum/100);
}

